package com.ruoyi.k3cloud.service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.kingdee.bos.webapi.sdk.K3CloudApi;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.k3cloud.business.prodOrder.constants.K3ProdField;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 金蝶生产订单Service
 * @author wjh
 * 2024/10/30
 */
@Service
@Slf4j
public class K3Service {

    @Autowired
    private K3CloudApi api;

    public List<JSONArray> selectList(String formId, List<String> fieldKeys, int startRow, int limit) {
        return selectList(formId, fieldKeys, startRow, limit, "", "");
    }

    /**
     * 查询列表
     */
    public List<JSONArray> selectList(String formId, List<String> fieldKeys, int startRow, int limit, String filterString, String orderString) {
        try {
            JSONObject params = new JSONObject();
            params.put("FormId", formId);
            params.put("FieldKeys", String.join(",", fieldKeys));
            params.put("StartRow", startRow);
            params.put("Limit", limit);
            params.put("FilterString", filterString);
            params.put("OrderString", orderString);
            params.put("TopRowCount", 0);
            params.put("SubSystemId", "");
            String res = api.executeBillQueryJson(JSONObject.toJSONString(params));
            if (StringUtils.isBlank(res)) {
                return Collections.emptyList();
            }
            return JSONArray.parseArray(res, JSONArray.class);
        } catch (Exception e) {
            log.error("获取云星空数据失败: formId = {}, message = {}", formId, e.getMessage());
            throw new ServiceException("获取云星空数据失败");
        }
    }
}
