package com.ruoyi.web.yh.report.controller;

import java.util.Collections;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.constants.LogTitle;
import com.ruoyi.common.core.validate.ValidGroup;
import com.ruoyi.common.enums.LogBizType;
import com.ruoyi.web.yh.report.domain.bo.ReportBO;
import com.ruoyi.web.yh.report.domain.dto.ReportVerifyDTO;
import com.ruoyi.web.yh.report.service.ReportAssembler;
import com.ruoyi.web.yh.report.service.ReportConverter;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.web.yh.report.domain.Report;
import com.ruoyi.web.yh.report.domain.ReportVO;
import com.ruoyi.web.yh.report.domain.ReportQuery;
import com.ruoyi.web.yh.report.service.ReportService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 报表Controller
 *
 * @author ruoyi
 * @date 2024-10-18
 */
@RestController
@RequestMapping("/yh/report")
public class ReportController extends BaseController
{
    @Autowired
    private ReportService reportService;

    @Autowired
    private ReportConverter reportConverter;

    @Autowired
    private ReportAssembler reportAssembler;

    private static final String LOG_TITLE = LogTitle.REPORT;

    /**
     * 查询报表列表
     */
    @PreAuthorize("@ss.hasAnyPermi({'yh:report:list', 'yh:report:groupList'})")
    @GetMapping("/list")
    public TableDataInfo list(ReportQuery query)
    {
        startPage();
        startOrderBy();
        List<ReportVO> list = reportService.selectReportList(query);
        return getDataTable(list);
    }

    /**
     * 导出报表列表
     */
    @PreAuthorize("@ss.hasPermi('yh:report:export')")
    @Log(title = LOG_TITLE, businessType = BusinessType.EXPORT, bizType = LogBizType.REPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReportQuery query)
    {
        List<ReportVO> list = reportService.selectReportList(query);
        ExcelUtil<ReportVO> util = new ExcelUtil<ReportVO>(ReportVO.class);
        util.exportExcel(response, list, "报表数据");
    }

    /**
     * 获取报表详细信息
     */
    @PreAuthorize("@ss.hasPermi('yh:report:query')")
    @GetMapping(value = "/{reportId}")
    public AjaxResult getInfo(@PathVariable("reportId") Long reportId,
                              @RequestParam(required = false, defaultValue = "true") Boolean needProductList,
                              @RequestParam(required = false, defaultValue = "true") Boolean needUserProd
    ) {
        ReportVO report = reportService.selectReportByReportId(reportId);
        List<ReportVO> list = Collections.singletonList(report);
        if (needProductList) {
            reportAssembler.assembleProductList(list, needUserProd);  // 拼接产量明细
        }
        return success(report);
    }

    /**
     * 新增报表
     */
    @PreAuthorize("@ss.hasPermi('yh:report:add')")
    @Log(title = LOG_TITLE, businessType = BusinessType.INSERT, bizType = LogBizType.REPORT, bizIdName = "arg0")
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) ReportVO data)
    {
        ReportBO bo = reportConverter.toBoByCreate(data);
        return toAjax(reportService.addReport(bo));
    }

    /**
     * 修改报表
     */
    @PreAuthorize("@ss.hasPermi('yh:report:edit')")
    @Log(title = LOG_TITLE, businessType = BusinessType.UPDATE, bizType = LogBizType.REPORT, bizIdName = "arg0")
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) ReportVO data) {
        ReportBO bo = reportConverter.toBoByUpdate(data);
        return toAjax(reportService.editReport(bo));
    }

    /**
     * 删除报表
     */
    @PreAuthorize("@ss.hasPermi('yh:report:remove')")
    @Log(title = LOG_TITLE, businessType = BusinessType.DELETE, bizType = LogBizType.REPORT, bizIdName = "arg0")
	@DeleteMapping("/{reportIds}")
    public AjaxResult remove(@PathVariable Long[] reportIds)
    {
        return toAjax(reportService.deleteReportByReportIds(reportIds));
    }

    @PreAuthorize("@ss.hasPermi('yh:report:verify')")
    @Log(title = LOG_TITLE, businessType = BusinessType.VERIFY, bizType = LogBizType.REPORT, bizIdName = "arg0")
	@PutMapping("/verify")
    public AjaxResult verify(@RequestBody @Validated ReportVerifyDTO dto) {
        return toAjax(reportService.verify(dto, getUserId()));
    }

    @PreAuthorize("@ss.hasPermi('yh:report:cancel')")
    @Log(title = LOG_TITLE, businessType = BusinessType.CANCEL, bizType = LogBizType.REPORT, bizIdName = "arg0")
    @PutMapping("/{reportId}/cancel")
    public AjaxResult cancel(@PathVariable Long reportId) {
        return toAjax(reportService.cancel(reportId));
    }

    @PreAuthorize("@ss.hasPermi('yh:report:submit')")
    @Log(title = LOG_TITLE, businessType = BusinessType.SUBMIT, bizType = LogBizType.REPORT, bizIdName = "arg0")
    @PutMapping("/{reportId}/submit")
    public AjaxResult submit(@PathVariable Long reportId) {
        return toAjax(reportService.submit(reportId));
    }
}
